@@ -199,37 +199,48 @@ class KarkkainenBlockwiseSA : public InorderBlockwiseSA<TStr> {
 public:
 	typedef DifferenceCoverSample<TStr> TDC;
 
-    KarkkainenBlockwiseSA(const TStr& __text,
-                          TIndexOffU __bucketSz,
-                          int __nthreads,
-                          uint32_t __dcV,
-                          uint32_t __seed = 0,
-                          bool __sanityCheck = false,
-                          bool __passMemExc = false,
-                          bool __verbose = false,
-                          string base_fname = "",
-                          ostream& __logger = cout) :
-    InorderBlockwiseSA<TStr>(__text, __bucketSz, __sanityCheck, __passMemExc, __verbose, __logger),
-    _sampleSuffs(EBWTB_CAT), _nthreads(__nthreads), _itrBucketIdx(0), _cur(0), _dcV(__dcV), _dc(EBWTB_CAT), _built(false), _base_fname(base_fname), _bigEndian(currentlyBigEndian())
+	KarkkainenBlockwiseSA(const TStr& __text,
+			      TIndexOffU __bucketSz,
+			      int __nthreads,
+			      uint32_t __dcV,
+			      uint32_t __seed = 0,
+			      bool __sanityCheck = false,
+			      bool __passMemExc = false,
+			      bool __verbose = false,
+			      string base_fname = "",
+			      ostream& __logger = cout) :
+		InorderBlockwiseSA<TStr>(__text, __bucketSz, __sanityCheck, __passMemExc, __verbose, __logger),
+		_sampleSuffs(EBWTB_CAT),
+		_nthreads(__nthreads),
+		_itrBucketIdx(0),
+		_cur(0),
+		_dcV(__dcV),
+		_dc(EBWTB_CAT),
+		_built(false),
+		_base_fname(base_fname),
+		_bigEndian(currentlyBigEndian()),
 #ifdef WITH_TBB
-,thread_group_started(false)
+		thread_group_started(false),
 #endif
+		_done(NULL)
     { _randomSrc.init(__seed); reset(); }
-    
+
     ~KarkkainenBlockwiseSA() throw()
     {
 #ifdef WITH_TBB
-		    tbb_grp.wait();
+	    tbb_grp.wait();
 #else
-        if(_threads.size() > 0) {
-            for (size_t tid = 0; tid < _threads.size(); tid++) {
-                _threads[tid]->join();
-                delete _threads[tid];
-            }
-        }
+	    if(_threads.size() > 0) {
+		    for (size_t tid = 0; tid < _threads.size(); tid++) {
+			    _threads[tid]->join();
+			    delete _threads[tid];
+		    }
+	    }
 #endif
+	    if (_done != NULL)
+		    delete[] _done;
     }
-    
+
     /**
      * Allocate an amount of memory that simulates the peak memory
      * usage of the DifferenceCoverSample with the given text and v.
@@ -258,9 +269,9 @@ public:
 #else
 			if(_threads.size() == 0) {
 #endif
-                _done = std::auto_ptr<volatile bool>(new volatile bool[_sampleSuffs.size() + 1]); 
+                _done = new volatile bool[_sampleSuffs.size() + 1];
                 for (size_t i = 0; i < _sampleSuffs.size() + 1; i++) {
-                    _done.get()[i] = false;
+                    _done[i] = false;
                 }
 				_itrBuckets.resize(this->_nthreads);
 				_tparams.resize(this->_nthreads);
@@ -293,7 +304,7 @@ public:
 				nextBlock((int)_cur);
 				_cur++;
 			} else {
-				while(!_done.get()[this->_itrBucketIdx]) {
+				while(!_done[this->_itrBucketIdx]) {
 					SLEEP(1);
 				}
 				// Read suffixes from a file
@@ -304,10 +315,10 @@ public:
 					cerr << "Could not open file for reading a suffix array: \"" << fname << "\"" << endl;
 					throw 1;
 				}
-				size_t numSAs = readU<TIndexOffU>(sa_file, _bigEndian);
+				size_t numSAs = readU<TIndexOffU>(sa_file, false /* do not endian swap */);
 				this->_itrBucket.resizeExact(numSAs);
 				for(size_t i = 0; i < numSAs; i++) {
-					this->_itrBucket[i] = readU<TIndexOffU>(sa_file, _bigEndian);
+					this->_itrBucket[i] = readU<TIndexOffU>(sa_file, false);
 				}
 				sa_file.close();
 				std::remove(fname.c_str());
@@ -373,7 +384,7 @@ public:
             }
             sa_file.close();
             sa->_itrBuckets[tid].clear();
-            sa->_done.get()[cur] = true;
+            sa->_done[cur] = true;
         }
     }
 #ifdef WITH_TBB
@@ -469,7 +480,7 @@ private:
 #endif
 	EList<pair<KarkkainenBlockwiseSA*, int> > _tparams;
 	ELList<TIndexOffU>      _itrBuckets;  /// buckets
-	std::auto_ptr<volatile bool>             _done;        /// is a block processed?
+	volatile bool *_done;        /// is a block processed?
 };
 
 
@@ -1126,7 +1137,7 @@ void KarkkainenBlockwiseSA<TStr>::nextBlock(int cur_block, int tid) {
                     if(ten > 0) VMSG_NL("  bucket " << (cur_block+1) << ": " << (ten * 10) << "%");
                 }
                 for(TIndexOffU i = iten; i < itenNext && i < len; i++) {
-                    assert_lt(jLo, (TIndexOff)i); assert_lt(jHi, (TIndexOff)i);
+                    assert_lt(jLo, (int64_t)i); assert_lt(jHi, (int64_t)i);
                     // Advance the upper-bound comparison by one character
                     if(i == hi || i == lo) continue; // equal to one of the bookends
                     if(hi != OFF_MASK && !suffixCmp(hi, i, jHi, kHi, kHiSoft, zHi)) {
