@@ -24,25 +24,18 @@
 #include <string>
 #include <stdexcept>
 #include <string.h>
+#include <fcntl.h>
 #include "sstring.h"
 
 #include "pat.h"
 #include "filebuf.h"
 #include "formats.h"
 #include "util.h"
+#include "str_util.h"
 #include "tokenize.h"
 
 using namespace std;
 
-/*void set_bufsz(unsigned bufsz) 
-{ 
-#if ZLIB_VERNUM < 0x1240        
-	std::fprintf(stderr, "Warning: gzbuffer added in zlib1.2.4. Unable to change buffer size from default of 8192.\n"); 
-#else        
-	gzbuffer(fp_, bufsz);
-#endif
-}*/
-
 /**
  * Calculate a per-read random seed based on a combination of
  * the read data (incl. sequence, name, quals) and the global
@@ -97,7 +90,7 @@ PatternSource* PatternSource::patsrcFromStrings(
 	const EList<string>& qs)
 {
 	switch(p.format) {
-		case FASTA:       return new FastaPatternSource(qs, p);
+		case FASTA:       return new FastaPatternSource(qs, p, p.interleaved);
 		case FASTA_CONT:  return new FastaContinuousPatternSource(qs, p);
 		case RAW:         return new RawPatternSource(qs, p);
 		case FASTQ:       return new FastqPatternSource(qs, p, p.interleaved);
@@ -106,6 +99,10 @@ PatternSource* PatternSource::patsrcFromStrings(
 		case TAB_MATE6:   return new TabbedPatternSource(qs, p, true);
 		case CMDLINE:     return new VectorPatternSource(qs, p);
 		case QSEQ:        return new QseqPatternSource(qs, p);
+#ifdef USE_SRA
+		case SRA_FASTA:
+		case SRA_FASTQ:   return new SRAPatternSource(qs, p);
+#endif
 		default: {
 			cerr << "Internal error; bad patsrc format: " << p.format << endl;
 			throw 1;
@@ -200,7 +197,7 @@ pair<bool, int> SoloPatternComposer::nextBatch(PerThreadReadBuf& pt) {
 			res = (*src_)[cur]->nextBatch(
 				pt,
 				true,  // batch A (or pairs)
-				true); // grab lock below
+				lock_); // grab lock below
 		} while(!res.first && res.second == 0);
 		if(res.second == 0) {
 			ThreadSafe ts(mutex_m);
@@ -230,7 +227,7 @@ pair<bool, int> DualPatternComposer::nextBatch(PerThreadReadBuf& pt) {
 			pair<bool, int> res = (*srca_)[cur]->nextBatch(
 				pt,
 				true,  // batch A (or pairs)
-				true); // grab lock below
+				lock_); // grab lock below
 			if(res.second == 0 && cur < srca_->size() - 1) {
 				ThreadSafe ts(mutex_m);
 				if(cur + 1 > cur_) cur_++;
@@ -293,6 +290,9 @@ PatternComposer* PatternComposer::setupPatternComposer(
 	const EList<string>& q,    // qualities associated with singles
 	const EList<string>& q1,   // qualities associated with m1
 	const EList<string>& q2,   // qualities associated with m2
+#ifdef USE_SRA
+	const EList<string>& sra_accs, // SRA accessions
+#endif
 	PatternParams& p,    // read-in parameters
 	bool verbose)              // be talkative?
 {
@@ -317,6 +317,24 @@ PatternComposer* PatternComposer::setupPatternComposer(
 		}
 	}
 
+#ifdef USE_SRA
+	for(size_t i = 0; i < sra_accs.size(); i++) {
+		const EList<string>* qs = &sra_accs;
+		EList<string> tmp;
+		if(p.fileParallel) {
+			// Feed query files one to each PatternSource
+			qs = &tmp;
+			tmp.push_back(sra_accs[i]);
+			assert_eq(1, tmp.size());
+		}
+		a->push_back(PatternSource::patsrcFromStrings(p, *qs));
+		b->push_back(NULL);
+		if(!p.fileParallel) {
+			break;
+		}
+	}
+#endif
+
 	// Create list of pattern sources for paired reads
 	for(size_t i = 0; i < m1.size(); i++) {
 		const EList<string>* qs = &m1;
@@ -456,24 +474,39 @@ void CFilePatternSource::open() {
 		if(infiles_[filecur_] == "-") {
 			// always assume that data from stdin is compressed
 			compressed_ = true;
-			int fn = dup(fileno(stdin));
-			zfp_ = gzdopen(fn, "rb");
+			int fd = dup(fileno(stdin));
+			zfp_ = gzdopen(fd, "rb");
+
+			if (zfp_ == NULL) {
+				close(fd);
+			}
 		}
 		else {
 			const char* filename = infiles_[filecur_].c_str();
-			const char* ext = filename + infiles_[filecur_].length();
 
-			// Only temporary until we have a separate BGZFPatternSource
-			while (ext != filename && *--ext != '.') ;
-			bool bam_file = ext != filename && strncmp(ext, ".bam", 4) == 0;
+			int fd = ::open(filename, O_RDONLY);
+			bool is_fifo = false;
 
-			if (!bam_file && is_gzipped_file(filename)) {
-				zfp_ = gzopen(filename, "rb");
+#ifndef _WIN32
+			struct stat st;
+			if (fstat(fd, &st) != 0) {
+				perror("stat");
+			}
+
+			is_fifo = S_ISFIFO(st.st_mode) != 0;
+#endif
+			if (pp_.format != BAM && (is_fifo || is_gzipped_file(fd))) {
+				zfp_ = gzdopen(fd, "r");
 				compressed_ = true;
 			} else {
-				fp_ = fopen(filename, "rb");
+				fp_ = fdopen(fd, "rb");
 			}
+
 			if((compressed_ && zfp_ == NULL) || (!compressed_ && fp_ == NULL)) {
+				if (fd != -1) {
+					close(fd);
+				}
+
 				if(!errs_[filecur_]) {
 					cerr << "Warning: Could not open read file \""
 					     << filename
@@ -485,17 +518,17 @@ void CFilePatternSource::open() {
 			}
 		}
 		is_open_ = true;
-        if (compressed_) {
+		if (compressed_) {
 #if ZLIB_VERNUM < 0x1235
-            cerr << "Warning: gzbuffer added in zlib v1.2.3.5. Unable to change "
-                    "buffer size from default of 8192." << endl;
+			cerr << "Warning: gzbuffer added in zlib v1.2.3.5. Unable to change "
+				"buffer size from default of 8192." << endl;
 #else
-            gzbuffer(zfp_, 64*1024);
+			gzbuffer(zfp_, 128*1024);
 #endif
-        }
-        else {
-            setvbuf(fp_, buf_, _IOFBF, 64*1024);
-        }
+		}
+		else {
+			setvbuf(fp_, buf_, _IOFBF, 64*1024);
+		}
 		return;
 	}
 	cerr << "Error: No input read files were valid" << endl;
@@ -684,7 +717,7 @@ pair<bool, int> FastaPatternSource::nextBatchFromFile(
 	bool batch_a, unsigned readi)
 {
 	int c;
-	EList<Read>& readbuf = batch_a ? pt.bufa_ : pt.bufb_;
+	EList<Read>* readbuf = batch_a ? &pt.bufa_ : &pt.bufb_;
 	if(first_) {
 		c = getc_wrapper();
 		if (c == EOF) {
@@ -701,8 +734,8 @@ pair<bool, int> FastaPatternSource::nextBatchFromFile(
 	}
 	bool done = false;
 	// Read until we run out of input or until we've filled the buffer
-	for(; readi < pt.max_buf_ && !done; readi++) {
-		Read::TBuf& buf = readbuf[readi].readOrigBuf;
+	while (readi < pt.max_buf_ && !done) {
+		Read::TBuf& buf = (*readbuf)[readi].readOrigBuf;
 		buf.clear();
 		buf.append('>');
 		while(true) {
@@ -713,9 +746,19 @@ pair<bool, int> FastaPatternSource::nextBatchFromFile(
 			}
 			buf.append(c);
 		}
+		if (interleaved_) {
+			// alternate between read buffers
+			batch_a = !batch_a;
+			readbuf = batch_a ? &pt.bufa_ : &pt.bufb_;
+			// increment read counter after each pair gets read
+			readi = batch_a ? readi+1 : readi;
+		} else {
+			readi++;
+		}
+
 	}
 	// Immediate EOF case
-	if(done && readbuf[readi-1].readOrigBuf.length() == 1) {
+	if(done && (*readbuf)[readi-1].readOrigBuf.length() == 1) {
 		readi--;
 	}
 	return make_pair(done, readi);
@@ -733,7 +776,7 @@ bool FastaPatternSource::parse(Read& r, Read& rb, TReadId rdid) const {
 	int c = -1;
 	size_t cur = 1;
 	const size_t buflen = r.readOrigBuf.length();
-	
+
 	// Parse read name
 	assert(r.name.empty());
 	while(cur < buflen) {
@@ -749,7 +792,7 @@ bool FastaPatternSource::parse(Read& r, Read& rb, TReadId rdid) const {
 	if(cur >= buflen) {
 		return false; // FASTA ended prematurely
 	}
-	
+
 	// Parse sequence
 	int nchar = 0;
 	assert(r.patFw.empty());
@@ -775,7 +818,7 @@ bool FastaPatternSource::parse(Read& r, Read& rb, TReadId rdid) const {
 	}
 	r.trimmed5 = (int)(nchar - r.patFw.length());
 	r.trimmed3 = (int)(r.patFw.trimEnd(pp_.trim3));
-	
+
 	for(size_t i = 0; i < r.patFw.length(); i++) {
 		r.qual.append('I');
 	}
@@ -912,9 +955,9 @@ bool FastaContinuousPatternSource::parse(
 
 	// Parse sequence
 	assert(ra.patFw.empty());
-	c = ra.readOrigBuf[cur++];
 	int nchar = 0;
 	while(cur < buflen) {
+		c = ra.readOrigBuf[cur++];
 		if(isalpha(c)) {
 			assert_in(toupper(c), "ACGTN");
 			if(nchar++ >= pp_.trim5) {
@@ -922,7 +965,6 @@ bool FastaContinuousPatternSource::parse(
 				ra.patFw.append(asc2dna[c]); // ascii to int
 			}
 		}
-		c = ra.readOrigBuf[cur++];
 	}
 	// record amt trimmed from 5' end due to --trim5
 	ra.trimmed5 = (int)(nchar - ra.patFw.length());
@@ -1148,83 +1190,24 @@ const uint8_t BAMPatternSource::EOF_MARKER[] = {
 	0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00,  0x00
 };
 
-bool BAMPatternSource::parse_bam_header() {
-	char magic[4];
-
-	if (zread(magic, 4) != 4 || strncmp(magic, "BAM\001", 4) != 0) {
-		std::cerr << "This file is not a BAM file" << std::endl;
-		return false;
-	}
-
-	int32_t l_text = 0;
-	int32_t n_ref  = 0;
-	char* data = NULL;
-	int32_t size = 0;
-
-	if (zread(&l_text, sizeof(l_text)) != sizeof(l_text)) {
-		return false;
-	}
-
-	size = l_text + 1;
-	data = new char[size];
-	if (data == NULL) {
-		return false;
-	}
-	if (zread(data, l_text) != l_text) {
-		delete[] data;
-		return false;
-	}
-	if (zread(&n_ref, sizeof(n_ref)) != sizeof(n_ref)) {
-		delete[] data;
-		return false;
-	}
-
-	for (int i = 0; i != n_ref; i++) {
-		int32_t l_name, l_ref;
-		if (zread(&l_name, sizeof(l_name)) != sizeof(l_name)) {
-			delete[] data;
-			return false;
-		}
-		if (l_name > size) {
-			size = l_name;
-			delete[] data;
-			data = new char[size];
-		}
-		if (zread(data, l_name) != l_name) {
-			delete[] data;
-			return false;
-		}
-		if (zread(&l_ref, sizeof(l_ref)) != sizeof(l_ref)) {
-			delete[] data;
-			return false;
-		}
-	}
-
-	delete[] data;
-
-	return true;
-}
-
 uint16_t BAMPatternSource::nextBGZFBlockFromFile(BGZF& b) {
-	fread(&b.hdr, sizeof(b.hdr), 1, fp_);
-	if (feof_unlocked(fp_) || ferror_unlocked(fp_)) {
-		return 0;
-	}
-
+        if (fread(&b.hdr, sizeof(b.hdr), 1, fp_) != 1) {
+		if (feof(fp_))
+			return 0;
+                std::cerr << "Error while reading BAM header" << std::endl;
+                exit(EXIT_FAILURE);
+        }
 	uint8_t *extra = new uint8_t[b.hdr.xlen];
-	fread(extra, b.hdr.xlen, 1, fp_);
-	if (ferror_unlocked(fp_)) {
-		return 0;
-	}
-
-	if (memcmp(EOF_MARKER, &b.hdr, sizeof(b.hdr)) == 0
-			&& memcmp(EOF_MARKER + sizeof(b.hdr), extra, 6 /* sizeof BAM subfield */) == 0)
+        if (fread(extra, b.hdr.xlen, 1, fp_) != 1) {
+                std::cerr << "Error while reading BAM extra subfields" << std::endl;
+                exit(EXIT_FAILURE);
+        }
+	if (memcmp(EOF_MARKER, &b.hdr, sizeof(b.hdr)) == 0 &&
+            memcmp(EOF_MARKER + sizeof(b.hdr), extra, 6 /* sizeof BAM subfield */) == 0)
 	{
 		delete[] extra;
-		fclose(fp_);
 		return 0;
 	}
-
 	uint16_t bsize = 0;
 	for (uint16_t i = 0; i < b.hdr.xlen;) {
 		if (extra[0] == 66 && extra[1] == 67) {
@@ -1236,81 +1219,75 @@ uint16_t BAMPatternSource::nextBGZFBlockFromFile(BGZF& b) {
 		uint16_t sub_field_len = *((uint16_t *)(extra + 2));
 		i = i + 2 + sub_field_len;
 	}
-
 	delete[] extra;
-
-	if (bsize == 0) {
+	if (bsize == 0)
 		return 0;
-	}
-
-	fread(b.cdata, bsize, 1, fp_);
-	if (ferror_unlocked(fp_)) {
-		return 0;
-	}
-
-	fread(&b.ftr, sizeof b.ftr, 1, fp_);
-	if (ferror_unlocked(fp_)) {
-		return 0;
-	}
-
+        if (fread(b.cdata, bsize, 1, fp_) != 1) {
+                std::cerr << "Error while reading BAM CDATA (compressed data)" << std::endl;
+                exit(EXIT_FAILURE);
+        }
+        if (fread(&b.ftr, sizeof(b.ftr), 1, fp_) != 1) {
+                std::cerr << "Error while reading BAM footer" << std::endl;
+                exit(EXIT_FAILURE);
+        }
 	return bsize;
 }
 
 std::pair<bool, int> BAMPatternSource::nextBatch(PerThreadReadBuf& pt, bool batch_a, bool lock) {
+        bool done = false;
 	uint16_t cdata_len;
-
 	unsigned nread = 0;
-	bool done = false;
-
-	pt.setReadId(readCnt_);
 
 	do {
 		if (bam_batch_indexes_[pt.tid_] >= bam_batches_[pt.tid_].size()) {
-			BGZF& block = blocks_[pt.tid_];
+			BGZF block;
 			std::vector<uint8_t>& batch = bam_batches_[pt.tid_];
 			if (lock) {
 				ThreadSafe ts(mutex);
 				if (first_) {
+                                        // parse the BAM header;
 					nextBGZFBlockFromFile(block);
 					first_ = false;
 				}
 				cdata_len = nextBGZFBlockFromFile(block);
 			} else {
+				if (first_) {
+					// parse the BAM header
+					nextBGZFBlockFromFile(block);
+					first_ = false;
+				}
 				cdata_len = nextBGZFBlockFromFile(block);
 			}
-
 			if (cdata_len == 0) {
-				ThreadSafe ts(orphan_mates_mutex_);
-				get_orphaned_pairs(pt.bufa_, pt.bufb_, pt.max_buf_, nread);
-				return make_pair(nread == 0, nread);
+				done = nread == 0;
+				break;
 			}
-
 			bam_batch_indexes_[pt.tid_] = 0;
-
 			batch.resize(block.ftr.isize);
-
 			int ret_code = decompress_bgzf_block(&batch[0], block.ftr.isize, block.cdata, cdata_len);
-
 			if (ret_code != Z_OK) {
 				return make_pair(true, 0);
 			}
-
 			uLong crc = crc32(0L, Z_NULL, 0);
 			crc = crc32(crc, &batch[0], batch.size());
 			assert(crc == block.ftr.crc32);
 		}
-
-		std::pair<bool, int> ret = get_alignments(pt, batch_a, nread);
-
+		std::pair<bool, int> ret = get_alignments(pt, batch_a, nread, lock);
 		done = ret.first;
 	} while (!done && nread < pt.max_buf_);
 
-	readCnt_ += 1;
-
+	if (lock) {
+		ThreadSafe ts(mutex);
+		pt.setReadId(readCnt_);
+		readCnt_ += nread;
+	} else {
+		pt.setReadId(readCnt_);
+		readCnt_ += nread;
+	}
 	return make_pair(done, nread);
 }
 
-std::pair<bool, int> BAMPatternSource::get_alignments(PerThreadReadBuf& pt, bool batch_a, unsigned& readi) {
+std::pair<bool, int> BAMPatternSource::get_alignments(PerThreadReadBuf& pt, bool batch_a, unsigned& readi, bool lock) {
 	size_t& i = bam_batch_indexes_[pt.tid_];
 	bool done = false;
 	bool read1 = true;
@@ -1343,14 +1320,19 @@ std::pair<bool, int> BAMPatternSource::get_alignments(PerThreadReadBuf& pt, bool
 			continue;
 		}
 
-		if (pp_.align_paired_reads && (((flag & 0x40) != 0
-				&& i + block_size == bam_batches_[pt.tid_].size())
-				|| ((flag & 0x80) != 0 && i == sizeof(block_size))))
+		if (pp_.align_paired_reads &&
+                    (((flag & 0x40) != 0 && i + block_size == bam_batches_[pt.tid_].size()) ||
+                     ((flag & 0x80) != 0 && i == sizeof(block_size))))
 		{
-			ThreadSafe ts(orphan_mates_mutex_);
-			store_orphan_mate(&bam_batches_[pt.tid_][0] + i, block_size);
-			i += block_size;
-			get_orphaned_pairs(pt.bufa_, pt.bufb_, pt.max_buf_, readi);
+			if (lock) {
+				ThreadSafe ts(orphan_mates_mutex_);
+				get_or_store_orhaned_mate(pt.bufa_, pt.bufb_, readi, &bam_batches_[pt.tid_][0] + i, block_size);
+				i += block_size;
+			} else {
+				get_or_store_orhaned_mate(pt.bufa_, pt.bufb_, readi, &bam_batches_[pt.tid_][0] + i, block_size);
+				i += block_size;
+			}
+
 		} else {
 			readbuf[readi].readOrigBuf.resize(block_size);
 
@@ -1358,102 +1340,63 @@ std::pair<bool, int> BAMPatternSource::get_alignments(PerThreadReadBuf& pt, bool
 			i += block_size;
 
 			read1 = !read1;
-			readi = (pp_.align_paired_reads
-					 && pt.bufb_[readi].readOrigBuf.length() == 0) ? readi : readi + 1;
+			readi = (pp_.align_paired_reads &&
+				 pt.bufb_[readi].readOrigBuf.length() == 0) ? readi : readi + 1;
 		}
 	}
 
 	return make_pair(done, readi);
 }
 
-void BAMPatternSource::store_orphan_mate(const uint8_t* r, size_t read_len) {
-	uint8_t flag;
-	memcpy(&flag, r + offset[BAMField::flag], sizeof(flag));
-
-	std::vector<orphan_mate_t>&
-		orphan_mates = (flag & 0x40) != 0 ? orphan_mate1s : orphan_mate2s;
-
+void BAMPatternSource::get_or_store_orhaned_mate(EList<Read>& buf_a, EList<Read>& buf_b, unsigned& readi, const uint8_t *mate, size_t mate_len) {
+	const char *read_name =
+		(const char *)(mate + offset[BAMField::read_name]);
 	size_t i;
-	for (i = 0; i < orphan_mates.size() && !orphan_mates[i].empty(); ++i) ;
+	uint32_t hash = hash_str(read_name);
+	orphan_mate_t *empty_slot = NULL;
 
+	for (i = 0; i < orphan_mates.size(); i++) {
+		if (empty_slot == NULL && orphan_mates[i].empty())
+			empty_slot = &orphan_mates[i];
+		if (orphan_mates[i].hash == hash)
+			break;
+	}
 	if (i == orphan_mates.size()) {
-		orphan_mates.resize(orphan_mates.size() * 2);
-	}
-
-	orphan_mate_t& mate = orphan_mates[i];
-
-	if (mate.data == NULL || mate.cap < read_len) {
-		mate.data = new uint8_t[read_len];
-		mate.cap = read_len;
-	}
-
-	if (mate.cap < read_len) {
-		mate.data = new uint8_t[read_len];
-		mate.cap = read_len;
-	}
-
-	mate.size = read_len;
-
-	memcpy(mate.data, r, read_len);
-}
-
-int BAMPatternSource::compare_read_names(const void* m1, const void* m2) {
-	const orphan_mate_t* mate1 = (const orphan_mate_t*)m1;
-	const orphan_mate_t* mate2 = (const orphan_mate_t*)m2;
-
-	const char* r1 = (const char *)(mate1->data + offset[BAMField::read_name]);
-	const char* r2 = (const char *)(mate2->data + offset[BAMField::read_name]);
-
-	return strcmp(r1, r2);
-}
-
-bool BAMPatternSource::compare_read_names2(const orphan_mate_t& m1, const orphan_mate_t& m2) {
-	if (m1.empty()) {
-		return false;
-	}
-
-	if (m2.empty()) {
-		return true;
-	}
-
-	const char* r1 = (const char *)(m1.data + offset[BAMField::read_name]);
-	const char* r2 = (const char *)(m2.data + offset[BAMField::read_name]);
-
-	return strcmp(r1, r2);
-}
-
-void BAMPatternSource::get_orphaned_pairs(EList<Read>& buf_a, EList<Read>& buf_b, const size_t max_buf, unsigned& readi) {
-	std::sort(orphan_mate1s.begin(), orphan_mate1s.end(), &BAMPatternSource::compare_read_names2);
-	std::sort(orphan_mate2s.begin(), orphan_mate2s.end(), &BAMPatternSource::compare_read_names2);
-
-	size_t lim1, lim2;
-	for (lim1 = 0; lim1 < orphan_mate1s.size() && !orphan_mate1s[lim1].empty(); lim1++) ;
-	for (lim2 = 0; lim2 < orphan_mate2s.size() && !orphan_mate2s[lim2].empty(); lim2++) ;
-
-	if (lim2 == 0) {
-		return;
-	}
-
-	for (size_t i = 0; i < lim1 && readi < max_buf; ++i) {
-		orphan_mate_t* mate1 = &orphan_mate1s[i];
-		orphan_mate_t* mate2 = (orphan_mate_t*)bsearch(mate1, &orphan_mate2s[0], lim2,
-		                            sizeof(orphan_mate2s[0]), compare_read_names);
-
-		if (mate2 != NULL) {
-			Read& ra = buf_a[readi];
-			Read& rb = buf_b[readi];
-
-			ra.readOrigBuf.resize(mate1->size);
-			rb.readOrigBuf.resize(mate2->size);
-
-			memcpy(ra.readOrigBuf.wbuf(), mate1->data, mate1->size);
-			memcpy(rb.readOrigBuf.wbuf(), mate2->data, mate2->size);
-
-			mate1->reset();
-			mate2->reset();
-
-			readi++;
+		// vector is full
+		if (empty_slot == NULL) {
+			orphan_mates.push_back(orphan_mate_t());
+			empty_slot = &orphan_mates.back();
 		}
+		empty_slot->hash = hash;
+		if (empty_slot->cap < mate_len) {
+			delete[] empty_slot->data;
+			empty_slot->data = NULL;
+		}
+		if (empty_slot->data == NULL) {
+			empty_slot->data = new uint8_t[mate_len];
+			empty_slot->cap = mate_len;
+		}
+		memcpy(empty_slot->data, mate, mate_len);
+		empty_slot->size = mate_len;
+	} else {
+		uint8_t flag;
+		Read& ra = buf_a[readi];
+		Read& rb = buf_b[readi];
+
+		memcpy(&flag, mate + offset[BAMField::flag], sizeof(flag));
+		if ((flag & 0x40) != 0) {
+			ra.readOrigBuf.resize(mate_len);
+			memcpy(ra.readOrigBuf.wbuf(), mate, mate_len);
+			rb.readOrigBuf.resize(orphan_mates[i].size);
+			memcpy(rb.readOrigBuf.wbuf(), orphan_mates[i].data, orphan_mates[i].size);
+		} else {
+			rb.readOrigBuf.resize(mate_len);
+			memcpy(rb.readOrigBuf.wbuf(), mate, mate_len);
+			ra.readOrigBuf.resize(orphan_mates[i].size);
+			memcpy(ra.readOrigBuf.wbuf(), orphan_mates[i].data, orphan_mates[i].size);
+		}
+		readi++;
+		orphan_mates[i].reset();
 	}
 }
 
@@ -1500,11 +1443,18 @@ bool BAMPatternSource::parse(Read& ra, Read& rb, TReadId rdid) const {
 	off += (l_seq+1)/2;
 	const char* qual = buf + off;
 	for (int i = 0; i < l_seq; i++) {
-		ra.qual.append(qual[i] + 33);
-		int base = "=ACMGRSVTWYHKDBN"[static_cast<uint8_t>(seq[i/2]) >> 4*(1-(i%2)) & 0xf];
-		ra.patFw.append(asc2dna[base]);
+		if (i < pp_.trim5) {
+			ra.trimmed5 += 1;
+		} else {
+			ra.qual.append(qual[i] + 33);
+			int base = "=ACMGRSVTWYHKDBN"[static_cast<uint8_t>(seq[i/2]) >> 4*(1-(i%2)) & 0xf];
+			ra.patFw.append(asc2dna[base]);
+		}
 	}
-	if (pp_.preserve_sam_tags) {
+	ra.trimmed3 = (int)(ra.patFw.trimEnd(pp_.trim3));
+	ra.qual.trimEnd(ra.trimmed3);
+
+	if (pp_.preserve_tags) {
 		off += l_seq;
 		ra.preservedOptFlags.install(buf + off, block_size - off);
 	}
@@ -1749,3 +1699,218 @@ void tooManyQualities(const BTString& read_name) {
 		 << "characters." << endl;
 	throw 1;
 }
+
+#ifdef USE_SRA
+
+std::pair<bool, int> SRAPatternSource::nextBatchImpl(
+	PerThreadReadBuf& pt,
+	bool batch_a)
+{
+	EList<Read>& readbuf = batch_a ? pt.bufa_ : pt.bufb_;
+	size_t readi = 0;
+	bool done = false;
+
+	for(; readi < pt.max_buf_; readi++) {
+		if(!sra_its_[pt.tid_]->nextRead() || !sra_its_[pt.tid_]->nextFragment()) {
+			done = true;
+			break;
+		}
+		const ngs::StringRef rname = sra_its_[pt.tid_]->getReadId();
+		const ngs::StringRef ra_seq = sra_its_[pt.tid_]->getFragmentBases();
+		const ngs::StringRef ra_qual = sra_its_[pt.tid_]->getFragmentQualities();
+		readbuf[readi].readOrigBuf.install(rname.data(), rname.size());
+		readbuf[readi].readOrigBuf.append('\t');
+		readbuf[readi].readOrigBuf.append(ra_seq.data(), ra_seq.size());
+		readbuf[readi].readOrigBuf.append('\t');
+		readbuf[readi].readOrigBuf.append(ra_qual.data(), ra_qual.size());
+		if(sra_its_[pt.tid_]->nextFragment()) {
+			const ngs::StringRef rb_seq = sra_its_[pt.tid_]->getFragmentBases();
+			const ngs::StringRef rb_qual = sra_its_[pt.tid_]->getFragmentQualities();
+			readbuf[readi].readOrigBuf.append('\t');
+			readbuf[readi].readOrigBuf.append(rb_seq.data(), rb_seq.size());
+			readbuf[readi].readOrigBuf.append('\t');
+			readbuf[readi].readOrigBuf.append(rb_qual.data(), rb_qual.size());
+		}
+		readbuf[readi].readOrigBuf.append('\n');
+	}
+
+	pt.setReadId(readCnt_);
+
+	{
+		ThreadSafe ts(mutex);
+		readCnt_ += readi;
+	}
+
+	return make_pair(done, readi);
+}
+
+/**
+ * TODO: need to think about whether this can be done in a sensible way
+ */
+std::pair<bool, int> SRAPatternSource::nextBatch(
+	PerThreadReadBuf& pt,
+	bool batch_a,
+	bool lock)
+{
+	if(lock) {
+		// synchronization at this level because both reading and manipulation of
+		// current file pointer have to be protected
+		return nextBatchImpl(pt, batch_a);
+	} else {
+		return nextBatchImpl(pt, batch_a);
+	}
+}
+
+/**
+ * Finalize tabbed parsing outside critical section.
+ */
+bool SRAPatternSource::parse(Read& ra, Read& rb, TReadId rdid) const {
+	// Light parser (nextBatchFromFile) puts unparsed data
+	// into Read& r, even when the read is paired.
+	assert(ra.empty());
+	assert(rb.empty());
+	assert(!ra.readOrigBuf.empty()); // raw data for read/pair is here
+	assert(rb.readOrigBuf.empty());
+	int c = '\t';
+	size_t cur = 0;
+	const size_t buflen = ra.readOrigBuf.length();
+
+	// Loop over the two ends
+	for(int endi = 0; endi < 2 && c == '\t'; endi++) {
+		Read& r = ((endi == 0) ? ra : rb);
+		assert(r.name.empty());
+		// Parse name if (a) this is the first end, or
+		// (b) this is tab6
+		if(endi < 1) {
+			// Parse read name
+			c = ra.readOrigBuf[cur++];
+			while(c != '\t' && cur < buflen) {
+				r.name.append(c);
+				c = ra.readOrigBuf[cur++];
+			}
+			assert_eq('\t', c);
+			if(cur >= buflen) {
+				return false; // record ended prematurely
+			}
+		} else if(endi > 0) {
+			// if this is the second end and we're parsing
+			// tab5, copy name from first end
+			rb.name = ra.name;
+		}
+
+		// Parse sequence
+		assert(r.patFw.empty());
+		c = ra.readOrigBuf[cur++];
+		int nchar = 0;
+		while(c != '\t' && cur < buflen) {
+			if(isalpha(c)) {
+				assert_in(toupper(c), "ACGTN");
+				if(nchar++ >= pp_.trim5) {
+					assert_neq(0, asc2dnacat[c]);
+					r.patFw.append(asc2dna[c]); // ascii to int
+				}
+			}
+			c = ra.readOrigBuf[cur++];
+		}
+		assert_eq('\t', c);
+		if(cur >= buflen) {
+			return false; // record ended prematurely
+		}
+		// record amt trimmed from 5' end due to --trim5
+		r.trimmed5 = (int)(nchar - r.patFw.length());
+		// record amt trimmed from 3' end due to --trim3
+		r.trimmed3 = (int)(r.patFw.trimEnd(pp_.trim3));
+
+		// Parse qualities
+		assert(r.qual.empty());
+		c = ra.readOrigBuf[cur++];
+		int nqual = 0;
+		if (pp_.intQuals) {
+			int cur_int = 0;
+			while(c != '\t' && c != '\n' && c != '\r' && cur < buflen) {
+				cur_int *= 10;
+				cur_int += (int)(c - '0');
+				c = ra.readOrigBuf[cur++];
+				if(c == ' ' || c == '\t' || c == '\n' || c == '\r') {
+					char cadd = intToPhred33(cur_int, pp_.solexa64);
+					cur_int = 0;
+					assert_geq(cadd, 33);
+					if(++nqual > pp_.trim5) {
+						r.qual.append(cadd);
+					}
+				}
+			}
+		} else {
+			while(c != '\t' && c != '\n' && c != '\r') {
+				if(c == ' ') {
+					wrongQualityFormat(r.name);
+					return false;
+				}
+				char cadd = charToPhred33(c, pp_.solexa64, pp_.phred64);
+				if(++nqual > pp_.trim5) {
+					r.qual.append(cadd);
+				}
+				if(cur >= buflen) break;
+				c = ra.readOrigBuf[cur++];
+			}
+		}
+		if(nchar > nqual) {
+			tooFewQualities(r.name);
+			return false;
+		} else if(nqual > nchar) {
+			tooManyQualities(r.name);
+			return false;
+		}
+		r.qual.trimEnd(pp_.trim3);
+		assert(c == '\t' || c == '\n' || c == '\r' || cur >= buflen);
+		assert_eq(r.patFw.length(), r.qual.length());
+	}
+	return true;
+}
+
+void SRAPatternSource::open() {
+	const string& sra_acc = sra_accs_[sra_acc_cur_];
+	string version = "bowtie2";
+	ncbi::NGS::setAppVersionString(version);
+	assert(!sra_acc.empty());
+	try {
+		// open requested accession using SRA implementation of the API
+		ngs::ReadCollection sra_run = ncbi::NGS::openReadCollection(sra_acc);
+
+		// compute window to iterate through
+		size_t MAX_ROW = sra_run.getReadCount();
+		pp_.upto -= pp_.skip;
+
+		if (pp_.upto <= MAX_ROW) {
+			MAX_ROW = pp_.upto;
+		}
+		if(MAX_ROW < 0) {
+			return;
+		}
+
+		size_t window_size = MAX_ROW / sra_its_.size();
+		size_t remainder = MAX_ROW % sra_its_.size();
+		size_t i = 0, start = 1;
+
+		if (pp_.skip > 0) {
+			start = pp_.skip + 1;
+			readCnt_ = pp_.skip;
+		}
+
+		while (i < sra_its_.size()) {
+			sra_its_[i] = new ngs::ReadIterator(sra_run.getReadRange(start, window_size, ngs::Read::all));
+			assert(sra_its_[i] != NULL);
+
+			i++;
+			start += window_size;
+			if (i == sra_its_.size() - 1) {
+				window_size += remainder;
+			}
+		}
+
+	} catch(...) {
+		cerr << "Warning: Could not access \"" << sra_acc << "\" for reading; skipping..." << endl;
+	}
+}
+
+#endif
