@@ -21,6 +21,8 @@
 #define PAT_H_
 
 #include <stdio.h>
+#include <stdint.h>
+#include <unistd.h>
 #include <sys/stat.h>
 #include <zlib.h>
 #include <cassert>
@@ -38,6 +40,14 @@
 #include "read.h"
 #include "util.h"
 
+#ifdef USE_SRA
+#include <ncbi-vdb/NGS.hpp>
+#include <ngs/ErrorMsg.hpp>
+#include <ngs/ReadCollection.hpp>
+#include <ngs/ReadIterator.hpp>
+#include <ngs/Read.hpp>
+#endif
+
 #ifdef _WIN32
 #define getc_unlocked _fgetc_nolock
 #endif
@@ -50,7 +60,6 @@
  * Parameters affecting how reads and read in.
  */
 struct PatternParams {
-	
 	PatternParams() { }
 
 	PatternParams(
@@ -68,9 +77,10 @@ struct PatternParams {
 		int sampleLen_,
 		int sampleFreq_,
 		size_t skip_,
+		uint64_t upto_,
 		int nthreads_,
 		bool fixName_,
-		bool preserve_sam_tags_,
+		bool preserve_tags_,
 		bool align_paired_reads_) :
 		format(format_),
 		interleaved(interleaved_),
@@ -86,28 +96,30 @@ struct PatternParams {
 		sampleLen(sampleLen_),
 		sampleFreq(sampleFreq_),
 		skip(skip_),
+		upto(upto_),
 		nthreads(nthreads_),
 		fixName(fixName_),
-		preserve_sam_tags(preserve_sam_tags_),
+		preserve_tags(preserve_tags_),
 		align_paired_reads(align_paired_reads_) { }
 
 	int format;			  // file format
-	bool interleaved;	  // some or all of the FASTQ reads are interleaved
+	bool interleaved;	  // some or all of the FASTQ/FASTA reads are interleaved
 	bool fileParallel;	  // true -> wrap files with separate PatternComposers
 	uint32_t seed;		  // pseudo-random seed
 	size_t max_buf;		  // number of reads to buffer in one read
 	bool solexa64;		  // true -> qualities are on solexa64 scale
 	bool phred64;		  // true -> qualities are on phred64 scale
 	bool intQuals;		  // true -> qualities are space-separated numbers
-	int trim5;            // amt to hard clip from 5' end
-	int trim3;            // amt to hard clip from 3' end
+	int trim5;                // amt to hard clip from 5' end
+	int trim3;                // amt to hard clip from 3' end
 	pair<short, size_t> trimTo;
 	int sampleLen;		  // length of sampled reads for FastaContinuous...
 	int sampleFreq;		  // frequency of sampled reads for FastaContinuous...
 	size_t skip;		  // skip the first 'skip' patterns
+	uint64_t upto;		  // max number of queries to read
 	int nthreads;		  // number of threads for locking
 	bool fixName;		  //
-	bool preserve_sam_tags; // keep existing SAM tags when aligning BAM files
+	bool preserve_tags;       // keep existing tags when aligning BAM files
 	bool align_paired_reads;
 };
 
@@ -115,7 +127,7 @@ struct PatternParams {
  * All per-thread storage for input read data.
  */
 struct PerThreadReadBuf {
-	
+
 	PerThreadReadBuf(size_t max_buf, int tid) :
 		max_buf_(max_buf),
 		bufa_(max_buf),
@@ -127,13 +139,13 @@ struct PerThreadReadBuf {
 		bufb_.resize(max_buf);
 		reset();
 	}
-	
+
 	Read& read_a() { return bufa_[cur_buf_]; }
 	Read& read_b() { return bufb_[cur_buf_]; }
-	
+
 	const Read& read_a() const { return bufa_[cur_buf_]; }
 	const Read& read_b() const { return bufb_[cur_buf_]; }
-	
+
 	/**
 	 * Return read id for read/pair currently in the buffer.
 	 */
@@ -141,7 +153,7 @@ struct PerThreadReadBuf {
 		assert_neq(rdid_, std::numeric_limits<TReadId>::max());
 		return rdid_ + cur_buf_;
 	}
-	
+
 	/**
 	 * Reset state as though no reads have been read.
 	 */
@@ -153,7 +165,7 @@ struct PerThreadReadBuf {
 		}
 		rdid_ = std::numeric_limits<TReadId>::max();
 	}
-	
+
 	/**
 	 * Advance cursor to next element
 	 */
@@ -161,7 +173,7 @@ struct PerThreadReadBuf {
 		assert_lt(cur_buf_, bufa_.size());
 		cur_buf_++;
 	}
-	
+
 	/**
 	 * Return true when there's nothing left for next().
 	 */
@@ -169,7 +181,7 @@ struct PerThreadReadBuf {
 		assert_leq(cur_buf_, bufa_.size());
 		return cur_buf_ >= bufa_.size()-1 || bufa_[cur_buf_+1].readOrigBuf.empty();
 	}
-	
+
 	/**
 	 * Just after a new batch has been loaded, use init to
 	 * set cur_buf_ appropriately.
@@ -177,14 +189,14 @@ struct PerThreadReadBuf {
 	void init() {
 		cur_buf_ = 0;
 	}
-	
+
 	/**
 	 * Set read id of first read in buffer.
 	 */
 	void setReadId(TReadId rdid) {
 		rdid_ = rdid;
 	}
-	
+
 	const size_t max_buf_; // max # reads to read into buffer at once
 	EList<Read> bufa_;	   // Read buffer for mate as
 	EList<Read> bufb_;	   // Read buffer for mate bs
@@ -205,16 +217,16 @@ extern void tooManyQualities(const BTString& read_name);
  * calls to lock() and unlock().
  */
 class PatternSource {
-	
+
 public:
-	
+
 	PatternSource(const PatternParams& p) :
 		pp_(p),
 		readCnt_(0),
 		mutex() { }
-	
+
 	virtual ~PatternSource() { }
-	
+
 	/**
 	 * Implementation to be provided by concrete subclasses.  An
 	 * implementation for this member is only relevant for formats
@@ -226,17 +238,17 @@ public:
 		PerThreadReadBuf& pt,
 		bool batch_a,
 		bool lock = true) = 0;
-	
+
 	/**
 	 * Finishes parsing a given read.  Happens outside the critical section.
 	 */
 	virtual bool parse(Read& ra, Read& rb, TReadId rdid) const = 0;
-	
+
 	/**
 	 * Reset so that next call to nextBatch* gets the first batch.
 	 */
 	virtual void reset() { readCnt_ = 0; }
-	
+
 	/**
 	 * Return a new dynamically allocated PatternSource for the given
 	 * format, using the given list of strings as the filenames to read
@@ -245,21 +257,21 @@ public:
 	static PatternSource* patsrcFromStrings(
 		const PatternParams& p,
 		const EList<std::string>& qs);
-	
+
 	/**
 	 * Return number of reads light-parsed by this stream so far.
 	 */
 	TReadId readCount() const { return readCnt_; }
-	
+
 protected:
-	
-	
+
+
 	// Reference to global input-parsing parameters
 	const PatternParams& pp_;
-	
+
 	// The number of reads read by this PatternSource
 	volatile TReadId readCnt_;
-	
+
 	// Lock enforcing mutual exclusion for (a) file I/O, (b) writing fields
 	// of this or another other shared object.
 	MUTEX_T mutex;
@@ -279,9 +291,9 @@ public:
 	VectorPatternSource(
 		const EList<std::string>& v,
 		const PatternParams& p);
-	
+
 	virtual ~VectorPatternSource() { }
-	
+
 	/**
 	 * Read next batch.  However, batch concept is not very applicable for this
 	 * PatternSource where all the info has already been parsed into the fields
@@ -292,7 +304,7 @@ public:
 		PerThreadReadBuf& pt,
 		bool batch_a,
 		bool lock = true);
-	
+
 	/**
 	 * Reset so that next call to nextBatch* gets the first batch.
 	 */
@@ -306,7 +318,7 @@ public:
 	 * Finishes parsing outside the critical section
 	 */
 	virtual bool parse(Read& ra, Read& rb, TReadId rdid) const;
-	
+
 private:
 
 	pair<bool, int> nextBatchImpl(
@@ -377,7 +389,7 @@ public:
 		PerThreadReadBuf& pt,
 		bool batch_a,
 		bool lock = true);
-	
+
 	/**
 	 * Reset so that next call to nextBatch* gets the first batch.
 	 * Should only be called by the master thread.
@@ -404,14 +416,19 @@ protected:
 	 * Reset state to handle a fresh file
 	 */
 	virtual void resetForNextFile() { }
-	
+
 	/**
 	 * Open the next file in the list of input files.
 	 */
 	void open();
 
 	int getc_wrapper() {
-		return compressed_ ? gzgetc(zfp_) : getc_unlocked(fp_);
+		int c;
+		do {
+			c = compressed_ ? gzgetc(zfp_) : getc_unlocked(fp_);
+		} while (c != EOF && c != '\t' && c != '\r' && c != '\n' && !isprint(c));
+
+		return c;
 	}
 
 	int ungetc_wrapper(int c) {
@@ -429,25 +446,29 @@ protected:
 		return r;
 	}
 
-	bool is_gzipped_file(const char* filename) {
-		FILE* f = fopen(filename, "rb");
-		if (ferror(f) > 0) {
+	bool is_gzipped_file(int fd) {
+		if (fd == -1) {
 			return false;
 		}
 
-		bool ret = false;
 		uint8_t byte1, byte2;
 
-		fread(&byte1, 1, sizeof(uint8_t), f);
-		fread(&byte2, 1, sizeof(uint8_t), f);
-		if (byte1 == 0x1f && byte2 == 0x8b) {
-			ret = true;
-		}
-		fclose(f);
+		ssize_t r1 = read(fd, &byte1, sizeof(uint8_t));
+		ssize_t r2 = read(fd, &byte2, sizeof(uint8_t));
 
-		return ret;
+		lseek(fd, 0, SEEK_SET);
+                if (r1 == 0 || r2 == 0) {
+                        std::cerr << "Unable to read file magic number" << std::endl;
+                        return false;
+                }
+
+		if (byte1 == 0x1f && byte2 == 0x8b) {
+			return true;
+		}
+
+		return false;
 	}
-	
+
 	EList<std::string> infiles_;	 // filenames for read files
 	EList<bool> errs_;		 // whether we've already printed an error for each file
 	size_t filecur_;		 // index into infiles_ of next file to read
@@ -472,13 +493,14 @@ private:
 class FastaPatternSource : public CFilePatternSource {
 
 public:
-	
+
 	FastaPatternSource(
 		const EList<std::string>& infiles,
-		const PatternParams& p) :
+		const PatternParams& p, bool interleaved) :
 		CFilePatternSource(infiles, p),
-		first_(true) { }
-	
+		first_(true),
+		interleaved_(interleaved) { }
+
 	/**
 	 * Reset so that next call to nextBatch* gets the first batch.
 	 * Should only be called by the master thread.
@@ -487,7 +509,7 @@ public:
 		first_ = true;
 		CFilePatternSource::reset();
 	}
-	
+
 	/**
 	 * Finalize FASTA parsing outside critical section.
 	 */
@@ -514,15 +536,16 @@ protected:
 		}
 		return c;
 	}
-	
+
 	/**
 	 * Reset state to handle a fresh file
 	 */
 	virtual void resetForNextFile() {
 		first_ = true;
 	}
-	
+
 	bool first_;
+	bool interleaved_;
 };
 
 /**
@@ -593,7 +616,7 @@ public:
 	virtual bool parse(Read& ra, Read& rb, TReadId rdid) const;
 
 protected:
-	
+
 	/**
 	 * Light-parse a batch into the given buffer.
 	 */
@@ -647,7 +670,7 @@ protected:
 		PerThreadReadBuf& pt,
 		bool batch_a,
 		unsigned read_idx);
-	
+
 	/**
 	 * Reset state to be read for the next file.
 	 */
@@ -692,7 +715,7 @@ public:
 		CFilePatternSource(infiles, p),
 		first_(true),
 		interleaved_(interleaved) { }
-	
+
 	virtual void reset() {
 		first_ = true;
 		CFilePatternSource::reset();
@@ -712,7 +735,7 @@ protected:
 		PerThreadReadBuf& pt,
 		bool batch_a,
 		unsigned read_idx);
-	
+
 	/**
 	 * Reset state to be ready for the next file.
 	 */
@@ -751,10 +774,12 @@ class BAMPatternSource : public CFilePatternSource {
 		orphan_mate_t() :
 			data(NULL),
 			size(0),
-			cap(0) {}
+			cap(0),
+			hash(0) {}
 
 		void reset() {
 			size = 0;
+			hash = 0;
 		}
 
 		bool empty() const {
@@ -764,6 +789,7 @@ class BAMPatternSource : public CFilePatternSource {
 		uint8_t* data;
 		uint16_t size;
 		uint16_t cap;
+		uint32_t hash;
 	};
 
 	struct BAMField {
@@ -790,11 +816,9 @@ public:
 		const PatternParams& p) :
 		CFilePatternSource(infiles, p),
 		first_(true),
-		blocks_(p.nthreads),
 		bam_batches_(p.nthreads),
 		bam_batch_indexes_(p.nthreads),
-		orphan_mate1s(p.nthreads * 2),
-		orphan_mate2s(p.nthreads * 2),
+		orphan_mates(p.nthreads * 2),
 		orphan_mates_mutex_(),
 		pp_(p) {
 			// uncompressed size of BGZF block is limited to 2**16 bytes
@@ -814,16 +838,9 @@ public:
 	virtual bool parse(Read& ra, Read& rb, TReadId rdid) const;
 
 	~BAMPatternSource() {
-		// only temporary until c++11
-		for (size_t i = 0; i < orphan_mate1s.size(); i++) {
-			if (orphan_mate1s[i].data != NULL) {
-				delete[] orphan_mate1s[i].data;
-			}
-		}
-
-		for (size_t i = 0; i < orphan_mate2s.size(); i++) {
-			if (orphan_mate2s[i].data != NULL) {
-				delete[] orphan_mate2s[i].data;
+		for (size_t i = 0; i < orphan_mates.size(); i++) {
+			if (orphan_mates[i].data != NULL) {
+				delete[] orphan_mates[i].data;
 			}
 		}
 	}
@@ -845,36 +862,23 @@ protected:
 	bool first_; // parsing first read in file
 
 private:
-
-	bool parse_bam_header();
-
 	virtual std::pair<bool, int> nextBatchFromFile(PerThreadReadBuf&, bool, unsigned) {
 		return make_pair(true, 0);
 	}
 
 	int decompress_bgzf_block(uint8_t *dst, size_t dst_len, uint8_t *src, size_t src_len);
-
-	std::pair<bool, int> get_alignments(PerThreadReadBuf& pt, bool batch_a, unsigned& readi);
-
+	std::pair<bool, int> get_alignments(PerThreadReadBuf& pt, bool batch_a, unsigned& readi, bool lock);
 	void store_orphan_mate(const uint8_t* read, size_t read_len);
-
 	void get_orphaned_pairs(EList<Read>& buf_a, EList<Read>& buf_b, const size_t max_buf, unsigned& readi);
-
+	void get_or_store_orhaned_mate(EList<Read>& buf_a, EList<Read>& buf_b, unsigned& readi, const uint8_t *mate, size_t mate_len);
 	size_t get_matching_read(const uint8_t* rec);
 
-	static int compare_read_names(const void* m1, const void* m2);
-
-	static bool compare_read_names2(const orphan_mate_t& m1, const orphan_mate_t& m2);
-
 	static const int offset[];
 	static const uint8_t EOF_MARKER[];
 
-	std::vector<BGZF> blocks_;
 	std::vector<std::vector<uint8_t> > bam_batches_;
 	std::vector<size_t> bam_batch_indexes_;
-
-	std::vector<orphan_mate_t> orphan_mate1s;
-	std::vector<orphan_mate_t> orphan_mate2s;
+	std::vector<orphan_mate_t> orphan_mates;
 	MUTEX_T orphan_mates_mutex_;
 
 	PatternParams pp_;
@@ -913,16 +917,16 @@ protected:
 		PerThreadReadBuf& pt,
 		bool batch_a,
 		unsigned read_idx);
-	
+
 	/**
 	 * Reset state to be ready for the next file.
 	 */
 	virtual void resetForNextFile() {
 		first_ = true;
 	}
-	
+
 private:
-	
+
 	bool first_;
 };
 
@@ -933,21 +937,21 @@ private:
 class PatternComposer {
 public:
 	PatternComposer(const PatternParams& p) : mutex_m() { }
-	
+
 	virtual ~PatternComposer() { }
-	
+
 	virtual void reset() = 0;
-	
+
 	/**
 	 * Member function override by concrete, format-specific classes.
 	 */
 	virtual std::pair<bool, int> nextBatch(PerThreadReadBuf& pt) = 0;
-	
+
 	/**
 	 * Make appropriate call into the format layer to parse individual read.
 	 */
 	virtual bool parse(Read& ra, Read& rb, TReadId rdid) = 0;
-	
+
 	/**
 	 * Given the values for all of the various arguments used to specify
 	 * the read and quality input, create a list of pattern sources to
@@ -961,13 +965,16 @@ public:
 		const EList<std::string>& q,	 // qualities associated with singles
 		const EList<std::string>& q1,	 // qualities associated with m1
 		const EList<std::string>& q2,	 // qualities associated with m2
+#ifdef USE_SRA
+		const EList<string>& sra_accs,   // SRA accessions
+#endif
 		PatternParams& p,		// read-in params
 		bool verbose);				// be talkative?
-	
+
 protected:
-	
+
 	static void free_EList_pmembers(const EList<PatternSource*>&);
-	
+
 	/// Lock enforcing mutual exclusion for (a) file I/O, (b) writing fields
 	/// of this or another other shared object.
 	MUTEX_T mutex_m;
@@ -978,9 +985,9 @@ protected:
  * unpaired reads, where the paired-end must come from parallel files.
  */
 class SoloPatternComposer : public PatternComposer {
-	
+
 public:
-	
+
 	SoloPatternComposer(
 		const EList<PatternSource*>* src,
 		const PatternParams& p) :
@@ -989,16 +996,17 @@ public:
 		src_(src)
 	{
 		assert(src_ != NULL);
+		lock_ = p.nthreads > 1;
 		for(size_t i = 0; i < src_->size(); i++) {
 			assert((*src_)[i] != NULL);
 		}
 	}
-	
+
 	virtual ~SoloPatternComposer() {
 		free_EList_pmembers(*src_);
 		delete src_;
 	}
-	
+
 	/**
 	 * Reset this object and all the PatternSources under it so that
 	 * the next call to nextBatch gets the very first read.
@@ -1009,7 +1017,7 @@ public:
 		}
 		cur_ = 0;
 	}
-	
+
 	/**
 	 * Calls member functions of the individual PatternSource objects
 	 * to get more reads.  Since there's no need to keep two separate
@@ -1027,6 +1035,7 @@ public:
 	}
 
 protected:
+	volatile bool lock_;
 	volatile size_t cur_; // current element in parallel srca_, srcb_ vectors
 	const EList<PatternSource*>* src_; /// PatternSources for paired-end reads
 };
@@ -1036,9 +1045,9 @@ protected:
  * unpaired reads, where the paired-end must come from parallel files.
  */
 class DualPatternComposer : public PatternComposer {
-	
+
 public:
-	
+
 	DualPatternComposer(
 		const EList<PatternSource*>* srca,
 		const EList<PatternSource*>* srcb,
@@ -1049,6 +1058,7 @@ public:
 		assert(srcb_ != NULL);
 		// srca_ and srcb_ must be parallel
 		assert_eq(srca_->size(), srcb_->size());
+		lock_ = p.nthreads > 1;
 		for(size_t i = 0; i < srca_->size(); i++) {
 			// Can't have NULL first-mate sources.	Second-mate sources
 			// can be NULL, in the case when the corresponding first-
@@ -1059,14 +1069,14 @@ public:
 			}
 		}
 	}
-	
+
 	virtual ~DualPatternComposer() {
 		free_EList_pmembers(*srca_);
 		delete srca_;
 		free_EList_pmembers(*srcb_);
 		delete srcb_;
 	}
-	
+
 	/**
 	 * Reset this object and all the PatternSources under it so that
 	 * the next call to nextBatch gets the very first read.
@@ -1080,7 +1090,7 @@ public:
 		}
 		cur_ = 0;
 	}
-	
+
 	/**
 	 * Calls member functions of the individual PatternSource objects
 	 * to get more reads.  Since we need to keep the two separate
@@ -1098,7 +1108,8 @@ public:
 	}
 
 protected:
-	
+
+	volatile bool lock_;
 	volatile size_t cur_; // current element in parallel srca_, srcb_ vectors
 	const EList<PatternSource*>* srca_; // for 1st matesunpaired
 	const EList<PatternSource*>* srcb_; // for 2nd mates
@@ -1112,9 +1123,9 @@ protected:
  * is thread-safe.
  */
 class PatternSourcePerThread {
-	
+
 public:
-	
+
 	PatternSourcePerThread(
 		PatternComposer& composer,
 		const PatternParams& pp, int tid) :
@@ -1123,21 +1134,21 @@ public:
 		pp_(pp),
 		last_batch_(false),
 		last_batch_size_(0) { }
-	
+
 	/**
 	 * Use objects in the PatternSource and/or PatternComposer
 	 * hierarchies to populate the per-thread buffers.
 	 */
 	std::pair<bool, bool> nextReadPair();
-	
+
 	Read& read_a() { return buf_.read_a(); }
 	Read& read_b() { return buf_.read_b(); }
-	
+
 	const Read& read_a() const { return buf_.read_a(); }
 	const Read& read_b() const { return buf_.read_b(); }
-	
+
 private:
-	
+
 	/**
 	 * When we've finished fully parsing and dishing out reads in
 	 * the current batch, we go get the next one by calling into
@@ -1149,7 +1160,7 @@ private:
 		buf_.init();
 		return res;
 	}
-	
+
 	/**
 	 * Once name/sequence/qualities have been parsed for an
 	 * unpaired read, set all the other key fields of the Read
@@ -1163,7 +1174,7 @@ private:
 	 * structs.
 	 */
 	void finalizePair(Read& ra, Read& rb);
-	
+
 	/**
 	 * Call into composition layer (which in turn calls into
 	 * format layer) to parse the read.
@@ -1211,14 +1222,14 @@ public:
 		composer_(composer),
 		pp_(pp),
 		tid_(tid) { }
-	
+
 	/**
 	 * Create a new heap-allocated PatternSourcePerThreads.
 	 */
 	virtual PatternSourcePerThread* create() const {
 		return new PatternSourcePerThread(composer_, pp_, tid_);
 	}
-	
+
 	/**
 	 * Create a new heap-allocated vector of heap-allocated
 	 * PatternSourcePerThreads.
@@ -1231,7 +1242,9 @@ public:
 		}
 		return v;
 	}
-	
+
+	virtual ~PatternSourcePerThreadFactory() {}
+
 private:
 	/// Container for obtaining paired reads from PatternSources
 	PatternComposer& composer_;
@@ -1239,4 +1252,101 @@ private:
 	int tid_;
 };
 
+#ifdef USE_SRA
+
+namespace ngs {
+	class ReadCollection;
+	class ReadIterator;
+}
+
+/**
+ * Pattern source for reading directly from the SRA archive.
+ */
+class SRAPatternSource : public PatternSource {
+public:
+	SRAPatternSource(
+		const EList<string>& sra_accs,
+		const PatternParams& p) :
+		PatternSource(p),
+		sra_accs_(sra_accs),
+		sra_acc_cur_(0),
+		cur_(0),
+		first_(true),
+		sra_its_(p.nthreads),
+		mutex_m(),
+		pp_(p)
+	{
+		assert_gt(sra_accs_.size(), 0);
+		errs_.resize(sra_accs_.size());
+		errs_.fill(0, sra_accs_.size(), false);
+		open(); // open first file in the list
+		sra_acc_cur_++;
+	}
+
+	virtual ~SRAPatternSource() {
+		for (size_t i = 0; i < sra_its_.size(); i++) {
+			if(sra_its_[i] != NULL) {
+				delete sra_its_[i];
+				sra_its_[i] = NULL;
+			}
+		}
+	}
+
+	/**
+	 * Fill Read with the sequence, quality and name for the next
+	 * read in the list of read files.	This function gets called by
+	 * all the search threads, so we must handle synchronization.
+	 *
+	 * Returns pair<bool, int> where bool indicates whether we're
+	 * completely done, and int indicates how many reads were read.
+	 */
+	virtual std::pair<bool, int> nextBatch(
+		PerThreadReadBuf& pt,
+		bool batch_a,
+		bool lock);
+
+	/**
+	 * Finishes parsing outside the critical section
+	 */
+	virtual bool parse(Read& ra, Read& rb, TReadId rdid) const;
+
+	/**
+	 * Reset so that next call to nextBatch* gets the first batch.
+	 * Should only be called by the master thread.
+	 */
+	virtual void reset() {
+		PatternSource::reset();
+		sra_acc_cur_ = 0,
+		open();
+		sra_acc_cur_++;
+	}
+
+protected:
+
+	std::pair<bool, int> nextBatchImpl(
+		PerThreadReadBuf& pt,
+		bool batch_a);
+
+	/**
+	 * Open the next file in the list of input files.
+	 */
+	void open();
+
+	EList<string> sra_accs_; // filenames for read files
+	EList<bool> errs_;       // whether we've already printed an error for each file
+	size_t sra_acc_cur_;     // index into infiles_ of next file to read
+	size_t cur_;             // current read id
+	bool first_;
+
+	std::vector<ngs::ReadIterator*> sra_its_;
+
+	/// Lock enforcing mutual exclusion for (a) file I/O, (b) writing fields
+	/// of this or another other shared object.
+	MUTEX_T mutex_m;
+
+	PatternParams pp_;
+};
+
+#endif
+
 #endif /*PAT_H_*/
